<?php
/**
 * @author Acacia Dynamic
 * @copyright 2024 Acacia Dynamic
 * @license private
 */
require_once __DIR__ . '/../../Classes/Wp_caller.php';
require_once __DIR__ . '/../../Kernel/API/Entity/AutomatisationEntity.php';
require_once __DIR__ . '/../../Kernel/API/Entity/DemandeEntity.php';
require_once __DIR__ . '/../../Kernel/API/Manager/AutomatisationManager.php';
require_once __DIR__ . '/../../Kernel/API/Manager/DemandeManager.php';
require_once __DIR__ . '/../../Kernel/API/Manager/DigitalSignageException.php';
require_once __DIR__ . '/../../Kernel/API/Manager/GabaritManager.php';
require_once __DIR__ . '/../../Kernel/API/Manager/PlaylistManager.php';
require_once __DIR__ . '/../../Classes/CreationVideoSpecialDataProvider.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class AutomatisationHookControllerCreation
{
    private $wp_caller;

    private $demande_manager;

    private $gabarit_manager;

    /**
     * @param null $wp_caller
     */
    public function __construct()
    {
        $this->wp_caller = new Wp_caller();
        $this->demande_manager = new DemandeManager($this->wp_caller);
        $this->gabarit_manager = GabaritManager::instance();
    }

    /**
     * @param string $newProductStatus
     * @param string $oldProductStatus
     * @param WP_Post $product
     *
     * @return void
     */
    public function handleProductChangeStatus($newProductStatus, $oldProductStatus, $product)
    {
        if ($product->post_type == 'product' && $newProductStatus == 'publish' && $oldProductStatus != 'publish') { // a decommenter lors du passage en prod
            // if ( $product->post_type == "product" && $newProductStatus == "publish" ) {
            $automatisations = $this->getAutomatisation();
            foreach ($automatisations as $automatisation) {
                $this->automatisationJob($automatisation, $product);
            }
        }
    }

    /**
     * @return AutomatisationEntity[] array
     */
    private function getAutomatisation()
    {
        /**         * @var AutomatisationEntity[] $allAutomatisations */
        $allAutomatisations = ($this->wp_caller->get_data(Wp_caller::AUTOMATISATIONS)) ?: [];

        return array_filter(
            $allAutomatisations,
            function ($automatisation) {
                $isHookCreation = filter_var($automatisation->getHookCreation(), FILTER_VALIDATE_BOOLEAN);
                $isActive = filter_var($automatisation->isActive(), FILTER_VALIDATE_BOOLEAN);

                return $isHookCreation && $isActive;
            }
        );
    }

    /**
     * @param AutomatisationEntity $automatisation
     * @param WP_Post $wordPressPost
     *
     * @return bool
     */
    public function automatisationJob($automatisation, $wordPressPost)
    {
        $wooCommerceProduct = new WC_Product($wordPressPost->ID);

        $priceIsGood = $this->checkAutomatisationPriceFilter($automatisation, $wooCommerceProduct);
        $categorieIsGood = $this->checkAutomatisationCategories($automatisation, $wooCommerceProduct);

        if ($priceIsGood && $categorieIsGood) {
            $videoAlreadyCreated = $this->demande_manager->getLocalDemandes('SELECT * FROM %s WHERE `gabarit` = ' . $automatisation->getGabarit() . ' AND `product_id` = ' . $wooCommerceProduct->get_id() . ' ORDER BY `id_demande` DESC LIMIT 1;');
            if (!empty($videoAlreadyCreated)) {
                $this->addDemandeInPlaylist($videoAlreadyCreated[0], $automatisation->getPlaylist());
                $this->wp_caller->save_data(Wp_caller::AUTOMATISATION_TRACE, date('Y-m-d H:i:s'));
            } else {
                $datasForPostDemande = $this->getDatasForPostDemande($automatisation, $wooCommerceProduct, $wordPressPost);
                $postDemandeResponse = $this->postVideoToApi($datasForPostDemande);

                if ($postDemandeResponse && $this->addDemandeInPlaylist($postDemandeResponse, $automatisation->getPlaylist())) {
                    $automatisation->setCreated($automatisation->getCreated() + 1);
                    AutomatisationManager::set($this->wp_caller)->update_automatisation($automatisation);
                    $this->wp_caller->save_data(Wp_caller::AUTOMATISATION_TRACE, date('Y-m-d H:i:s'));
                }
            }
        }
    }

    /**
     * @param AutomatisationEntity $automatisation
     * @param WC_Product $wooCommerceProduct
     *
     * @return bool
     */
    private function checkAutomatisationPriceFilter($automatisation, $wooCommerceProduct)
    {
        $autoPriceMin = (int) $automatisation->getPriceMin();
        $autoPriceMax = (int) $automatisation->getPriceMax();

        if ($autoPriceMin == -1 && $autoPriceMax == -1) {
            return true;
        } elseif ($autoPriceMin >= 0 && $autoPriceMax == -1) {
            return $wooCommerceProduct->get_price() >= $autoPriceMin;
        } elseif ($autoPriceMin >= 0 && $autoPriceMax >= 0) {
            return $wooCommerceProduct->get_price() >= $autoPriceMin && $wooCommerceProduct->get_price() <= $autoPriceMax;
        }

        return false;
    }

    /**
     * @param AutomatisationEntity $automatisation
     * @param WC_Product $wooCommerceProduct
     *
     * @return bool
     */
    private function checkAutomatisationCategories($automatisation, $wooCommerceProduct)
    {
        $productCategories = $wooCommerceProduct->get_category_ids();
        $automatisationCategorie = $automatisation->getCategorie();
        $categorieMatch = empty($automatisationCategorie); // Si aucune categorie n'est set dans l'automatisation alors le match par defaut et a true
        foreach ($productCategories as $productCategorie) {
            if (in_array($productCategorie, $automatisationCategorie)) {
                $categorieMatch = true;
            }
        }

        return $categorieMatch;
    }

    /**
     * @param DemandeEntity $demande
     * @param $playlistId
     *
     * @return bool
     */
    private function addDemandeInPlaylist($demande, $playlistId)
    {
        try {
            $playlistManager = new PlaylistManager($this->wp_caller);
            $playlist = $playlistManager->get_playlist($playlistId);
            if ($playlist) {
                $videosPlaylist = $playlist->get_videos();
                $videoPlaylist = $this->removeArrayValue($videosPlaylist, $demande->getIdDemande());
                array_unshift($videoPlaylist, $demande->getIdDemande());
                $playlist->set_videos($videoPlaylist);
                $playlistManager->update_playlist($playlist);
            }

            return true;
        } catch (DigitalSignageException $e) {
            return false;
        }
    }

    private function removeArrayValue($array, $value)
    {
        if (($key = array_search($value, $array)) !== false) {
            unset($array[$key]);
        }

        return $array;
    }

    /**
     * @param array $datas
     *
     * @return DemandeEntity|false
     *                             id_lot = id_lot+1
     */
    public function postVideoToApi($datas, $id_lot = 1)
    {
        $video = new DemandeEntity();

        $video->setData($datas);
        $video->setGabarit($datas['gabarit']);
        $video->setDateBegin(date('y-m-d H:i:s'));
        $video->setDateEnd(date('y-m-d H:i:s'));
        $video->setVideoComment('');
        $video->setVideoName((isset($datas['produit_libelle'])) ? $datas['produit_libelle'] : DigitalSignageTranslate::getTraduction('Default_video_name'));
        $video->setAutomatique($id_lot);
        $video->setDateDemande(date('Y-m-d H:i:s'));
        $video->setProductId($datas['product_id']);
        $video->setDownloaded('0');

        // TODO
        $specialChampsManager = new CreationVideoSpecialDataProvider();
        $data_youtube = $specialChampsManager->getyoutubevideo($datas['product_id']);
        /*
        $data_youtube=array();
        $data_youtube['titleVideo']='titre de la video';
        $data_youtube['summaryVideo']='description de la video';
        $data_youtube['tagsVideo']='tag1,tag2';
        */
        $video->setYoutubeData(json_encode($data_youtube));
        // $video->setYoutubeData($data_youtube);

        // poste la création de video
        try {
            $demande = $this->demande_manager->postDemande($video);
            if ($demande) {
                if ($this->wp_caller->execute_video_insert(
                    'INSERT INTO `%s` (`id_demande`,                       `data`,                        `flag`,                        `video_name`,                      `video_comment`,                       `date_begin`,                            `automatique`,                        `date_end`,                       `date_demande`,                       `gabarit`,                        `video_url`,                       `video_hash`,                       `product_id`,`youtube_data`) VALUES
                                               (' . $demande->getIdDemande() . ',        "' . pSQL($demande->getData()) . '", "' . $demande->getFlag() . '", "' . $demande->getVideoName() . '", "' . $demande->getVideoComment() . '", "' . $demande->getDateBegin() . '", "' . $demande->getAutomatique() . '", "' . $demande->getDateEnd() . '", "' . $demande->getDateDemande() . '", "' . $demande->getGabarit() . '", "' . $demande->getVideoUrl() . '", "' . $demande->getVideoHash() . '", "' . $demande->getProductId() . '", "' . pSQL($demande->getYoutubeData()) . '") '
                )) {
                    return $demande;
                }
            }

            return false;
        } catch (DigitalSignageException $e) {
            return false;
        }
    }

    /**
     * @param AutomatisationEntity $automatisation
     * @param WC_Product $wooCommerceProduct
     * @param WP_Post $wordPressPost
     *
     * @return array
     */
    public function getDatasForPostDemandeAvecGabarit($Gabarit_param, $wooCommerceProduct, $wordPressPost)
    {
        // file_put_contents('debug_ag.txt','getDatasForPostDemandeAvecGabarit'."\r\n",FILE_APPEND);
        // detecter si personnalisation du gabarit
        // si oui modifie le datasForPostDemande!!!!!
        $gabarit = $this->gabarit_manager->getGabarit($Gabarit_param);

        $datas = [];
        $imageCompteur = 1;
        // foreach ($gabarit->getChamps() as $champ) {
        $champs__ = $gabarit->initCustomChamp();
        foreach ($champs__ as $champ) {
            if ($champ['type'] == 'picture') {
                if (strpos($champ['default'], ';') !== false) {// AG je ne peux pas faire autrement
                    $datas[$champ['name']] = $champ['default'];
                    $tab_temp = [];
                    $tab_temp = explode(';', $champ['default']);
                    if (count($tab_temp) >= 2) {
                        $datas[$champ['name']] = $tab_temp[1];
                    }
                } else {
                    if ($champ['default'] !== '') {
                        // TODO il faut detecter les mots clefs images!!! ex: product01PictureUrl01 !!!
                        // file_put_contents('debug_ag.txt',$champ['name'].' '.$champ['default']."\r\n",FILE_APPEND);
                        $specialChampsManager = new CreationVideoSpecialDataProvider();
                        $callableFunctionData = [$specialChampsManager, $champ['default']];
                        if (is_callable($callableFunctionData)) {
                            $tab_temp = [];
                            $tab_temp = explode(';', call_user_func($callableFunctionData));
                            // $datas[$champ['name']] = call_user_func($callableFunctionData);
                            $datas[$champ['name']] = $this->getDatasPictureForPostDemande($wooCommerceProduct, $this->get_number_picture_from_name($champ['default']));
                            if (count($tab_temp) >= 2) {
                                $datas[$champ['name']] = $tab_temp[1];
                            }
                        } else {
                            $datas[$champ['name']] = $this->getDatasPictureForPostDemande($wooCommerceProduct, $this->get_number_picture_from_name($champ['default']));
                        }
                    } else {
                        $datas[$champ['name']] = $this->getDatasPictureForPostDemande($wooCommerceProduct, $imageCompteur);
                        ++$imageCompteur;
                    }
                }
            } else {
                $champ_name = $champ['wordpress_champ_name'];

                if ($champ_name == 'IA_description') {
                    // il faut pousser dans $datas[$champ['name']]
                    // le retour de l'api AI
                    $specialChampsManager = new CreationVideoSpecialDataProvider();
                    $datas[$champ['name']] = $specialChampsManager->IA_description($champ['ia_question'], $wooCommerceProduct->get_id());
                } else {
                    $datas[$champ['name']] = $this->get_data_from_champ_name($wooCommerceProduct, $champ_name, $champ['default']);
                }
            }
        }

        $datas['gabarit'] = $Gabarit_param;
        $datas['produit_libelle'] = $wooCommerceProduct->name;
        $datas['product_id'] = $wooCommerceProduct->id;

        return $datas;
    }

    private function get_data_from_champ_name($wooCommerceProduct, $champ_name, $default)
    {
        $retour = $default;
        $callableFunctionData = [$wooCommerceProduct, 'get_' . $champ_name];
        if (is_callable($callableFunctionData)) {
            $retour = call_user_func($callableFunctionData);
        } else {
            if ($champ_name == 'first_category') {
                $terms = get_the_terms($wooCommerceProduct->get_id(), 'product_cat');
                if ($terms && !is_wp_error($terms)) {
                    $retour = $terms[0]->name;
                } else {
                    $retour = '';
                }
            } else {
                if ($champ_name == 'pourcent_reduc') {
                    $retour = '';
                    $productDatas = $wooCommerceProduct->get_data();
                    if ($productDatas['regular_price'] == $productDatas['price']) {
                        $retour = '0%';
                    } else {
                        if ($productDatas['regular_price'] > 0) {
                            $retour = '-' . round(($productDatas['price'] * 100) / $productDatas['regular_price']) . '%';
                        }
                    }
                } else {
                    $specialChampsManager = new CreationVideoSpecialDataProvider();
                    $callableFunctionData = [$specialChampsManager, $champ_name];
                    if (is_callable($callableFunctionData)) {
                        $retour = call_user_func($callableFunctionData);
                    }
                }
            }
        }

        return $retour;
    }

    /**
     * @param AutomatisationEntity $automatisation
     * @param WC_Product $wooCommerceProduct
     * @param WP_Post $wordPressPost
     *
     * @return array
     */
    private function getDatasForPostDemande($automatisation, $wooCommerceProduct, $wordPressPost)
    {
        // file_put_contents('debug_ag.txt','getDatasForPostDemande'."\r\n",FILE_APPEND);
        $gabarit = $this->gabarit_manager->getGabarit($automatisation->getGabarit());
        $datas = [];
        $imageCompteur = 1;
        $champs__ = $gabarit->initCustomChamp();
        // foreach ($gabarit->getChamps() as $champ) {
        foreach ($champs__ as $champ) {
            if ($champ['type'] == 'picture') {
                if (strpos($champ['default'], ';') !== false) {// AG je ne peux pas faire autrement
                    $datas[$champ['name']] = $champ['default'];
                    $tab_temp = [];
                    $tab_temp = explode(';', $champ['default']);
                    if (count($tab_temp) >= 2) {
                        $datas[$champ['name']] = $tab_temp[1];
                    }
                } else {
                    // il faut detecter les mots clefs images!!! ex: product01PictureUrl01 !!!
                    if ($champ['default'] !== '') {
                        // il faut detecter les mots clefs images!!! ex: product01PictureUrl01 !!!
                        $specialChampsManager = new CreationVideoSpecialDataProvider();
                        $callableFunctionData = [$specialChampsManager, $champ['default']];
                        if (is_callable($callableFunctionData)) {
                            $tab_temp = [];
                            $tab_temp = explode(';', call_user_func($callableFunctionData));
                            // $datas[$champ['name']] = call_user_func($callableFunctionData);
                            $datas[$champ['name']] = $this->getDatasPictureForPostDemande($wooCommerceProduct, $this->get_number_picture_from_name($champ['default']));
                            if (count($tab_temp) >= 2) {
                                $datas[$champ['name']] = $tab_temp[1];
                            }
                        } else {
                            $datas[$champ['name']] = $this->getDatasPictureForPostDemande($wooCommerceProduct, $this->get_number_picture_from_name($champ['default']));
                        }
                    } else {
                        $datas[$champ['name']] = $this->getDatasPictureForPostDemande($wooCommerceProduct, $imageCompteur);
                        ++$imageCompteur;
                    }
                }
            } else {
                $champ_name = $champ['wordpress_champ_name'];

                if ($champ_name == 'IA_description') {
                    // il faut pousser dans $datas[$champ['name']]
                    // le retour de l'api AI
                    $specialChampsManager = new CreationVideoSpecialDataProvider();
                    $datas[$champ['name']] = $specialChampsManager->IA_description($champ['ia_question'], $wooCommerceProduct->get_id());
                } else {
                    $datas[$champ['name']] = $this->get_data_from_champ_name($wooCommerceProduct, $champ_name, $champ['default']);
                }
            }
        }

        $datas['gabarit'] = $automatisation->getGabarit();
        $datas['produit_libelle'] = $wooCommerceProduct->get_name();
        $datas['product_id'] = $wooCommerceProduct->get_id();

        return $datas;
    }

    private function get_number_picture_from_name($name_picture)
    {
        switch ($name_picture) {
            case 'product01PictureUrl01':
                return 1;
            case 'product02PictureUrl01':
                return 2;
            case 'product03PictureUrl01':
                return 3;
            case 'product04PictureUrl01':
                return 4;
            case 'product05PictureUrl01':
                return 5;
            case 'product06PictureUrl01':
                return 6;
            case 'product07PictureUrl01':
                return 7;
            case 'product08PictureUrl01':
                return 8;
            case 'product09PictureUrl01':
                return 9;
            case 'product10PictureUrl01':
                return 10;
            default:
                return 1;
        }
    }

    /**
     * cette function tourne en boucle, si une clé est superieur au nombre d'images alors elle renvois les images depuis le debut
     *
     * @param WC_Product $wooCommerceProduct
     * @param int $imageNumber
     *
     * @return string
     */
    private function getDatasPictureForPostDemande($prestaProduct, $imageNumber)
    {
        $imagesPaths = $this->getImages($prestaProduct->id);
        $nombreItems = count($imagesPaths);
        $imageNumber = $imageNumber - 1;
        $key = $imageNumber % $nombreItems;

        return $imagesPaths[$key]['path'];
    }

    public function getImages($id_product)
    {
        // Tous les lien vers les images
        $all_links = [];

        // Initialize the link object
        $link = new Link();

        // Récuperation du protocole
        $protocol = (isset($_SERVER['HTTPS'])) ? ($_SERVER['HTTPS'] && $_SERVER['HTTPS'] != 'off') ? 'https' : 'http' : 'http';

        // Language id
        $id_lang = (int) Configuration::get('PS_LANG_DEFAULT');

        // Load Product Object
        $product = new Product($id_product, false, $id_lang);

        // Validate CMS Page object
        if (true && Validate::isLoadedObject($product)) {
            // Get product images
            $productImages = $product->getImages((int) $id_lang);

            if ($productImages && count($productImages) > 0) {
                foreach ($productImages as $key => $val) {
                    // Récuperation de l'id du produit
                    $id_image = $val['id_image'];

                    // Convertion et ajout au images depuis l'id d'une image en son chemin relatif
                    $all_links[] = [
                        'url' => $protocol . '://' . $link->getImageLink($product->link_rewrite[Context::getContext()->language->id], $id_image, ImageType::getFormattedName('home')),
                        'path' => _PS_PROD_IMG_DIR_ . implode('/', str_split($id_image)) . '/' . $id_image . '.jpg',
                    ];
                }
            }
        }

        return $all_links;
    }
}
